rm(list = ls())
#------------ librarys ------------#
library(doParallel)
library(foreach)
library(assist)
library(wavethresh)
library(gss)

#------------ functions for methods ----------#

source("fit_only.R")

#------------ parameters set ------------#
registerDoParallel(cores = 20)
noise.seed <- 830
n <- 10000
nsim <- 100
nsub <- 1000
s <- 0.1
method <- "m" # change to "v" for gcv & save result in different directory
gss_alpha <- 1 # for gcv
sample.seed <- seq(138,len=nsim)

getDoParWorkers()

#------------ data generation -----------#
x <- seq(0,1,len=n)
f <- sin(32*pi*x)-8*(x-.5)^2
set.seed(noise.seed)
z <- matrix(f+s*rnorm(n*nsim),n,nsim)



#------------- result object -----------#
p1 <- p2 <- p3 <- p4 <- p5o10 <- p5o20 <-p5o30 <- p6 <- p7 <-NULL
time1 <- time2 <- time3 <- time4 <-time5o10 <- time5o20 <- time5o30 <-time6 <- time7 <-NULL



#------------ 1. ssr all ------------ #
for (i in 1:nsim){
  ptm <- proc.time()
  tmp <- ssr(z[,i]~x,cubic(x),spar=method)
  elapsed <-proc.time()-ptm
  p1 <- cbind(p1,tmp$fit)
  time1 <- c(time1,elapsed[3])
}

#------------ 2.rdr ------------ #
for (i in 1:nsim){
  print(i)
  ptm <- proc.time()
  p2 <- cbind(p2,rdr(x,z[,i],nsub,spar=method,seed=sample.seed[i]))
  elapsed<-proc.time()-ptm
  time2 <-c(time2,elapsed[3])
}

#------------ 3. drdr ------------ #
for (i in 1:nsim){
  print(i)
  ptm <- proc.time()
  p3 <- cbind(p3,drdr(x,z[,i],nsub,spar=method,seed=sample.seed[i]))
  elapsed<-proc.time()-ptm
  time3 <-c(time3,elapsed[3])
}

#------------ 4.sdr ------------ #
for (i in 1:nsim){
  print(i)
  ptm <- proc.time()
  p4 <- cbind(p4,sdr(x,z[,i],nsub,spar=method))
  elapsed<-proc.time()-ptm
  time4 <-c(time4,elapsed[3])
}

#------------ 5.osdr overlap 0.1------------ #
overlap.rate <- 0.1 # for osdr
for (i in 1:nsim){
  print(i)
  ptm <- proc.time()
  p5o10 <- cbind(p5o10,osdr(x,z[,i],nsub,overlap=overlap.rate,spar=method))
  elapsed<-proc.time()-ptm
  time5o10 <-c(time5o10,elapsed[3])
}

#------------ 5.osdr overlap 0.2------------ #
overlap.rate <- 0.2 # for osdr
for (i in 1:nsim){
  print(i)
  ptm <- proc.time()
  p5o20 <- cbind(p5o20,osdr(x,z[,i],nsub,overlap=overlap.rate,spar=method))
  elapsed<-proc.time()-ptm
  time5o20 <-c(time5o20,elapsed[3])
}

#------------ 5.osdr overlap 0.3------------ #
overlap.rate <- 0.3 # for osdr
for (i in 1:nsim){
  print(i)
  ptm <- proc.time()
  p5o30 <- cbind(p5o30,osdr(x,z[,i],nsub,overlap=overlap.rate,spar=method))
  elapsed<-proc.time()-ptm
  time5o30 <-c(time5o30,elapsed[3])
}


#------------ 6.ubs ------------#
result6 <-foreach(i=1:nsim,.combine=cbind) %dopar%{
  library(gss)
  ptm <- proc.time()
  fit <- ubs(x,z[,i],method=method,alpha=gss_alpha)
  elapsed <-proc.time()-ptm
  c(elapsed[3],fit)
}

time6 <-result6[1,]
p6 <-result6[-1,]


#------------ 7.abs ------------#
result7 <-foreach(i=1:nsim,.combine=cbind) %dopar%{
  library(gss)
  ptm <- proc.time()
  tmp <- abs(x,z[,i],alpha=gss_alpha,method=method)
  elapsed <-proc.time()-ptm
  c(elapsed[3],tmp)
}

time7 <-result7[1,]
p7 <-result7[-1,]


#----------- Save Result ----------#
save(x, f, p1, p2, p3, p4, p5o10, p5o20, p5o30, p6, p7,
     time1, time2, time3, time4, time5o10, time5o20, time5o30, time6, time7,
     file="sin_s1.rda")

#----- Compute average time elapsed -----#
sin.meantime=c(ALL=mean(time1),RDR=mean(time2),DRDR=mean(time3),SDR=mean(time4),
               OSDR10=mean(time5o10),OSDR20=mean(time5o20),OSDR30=mean(time5o30),
               UBS=mean(time6),ABS=mean(time7))
