##------- 2.rdr ---------##
rdr <- function(x,y,nsub,core=20,
                spar = "v",limnla = c(-10, 3), control = list(),seed=NULL){
  kmeansq <- function(x){
    sum(x)/(length(x))^2
  }
  registerDoParallel(cores = core)
  N <- length(x)
  k <- floor(N/nsub)	
  bounds <- data.frame(lb=c(1+(1:k-1)*nsub),ub=c((1:(k-1))*nsub,N))
  d <- data.frame(x=x, y=y)
  set.seed(seed)
  d <- d[sample(1:N),]
  output <- foreach(i=1:k,.combine = rbind) %dopar%{
    dsub <- d[(bounds[i,1]):(bounds[i,2]),]
    tmp <- ssr(y~x,cubic(x),spar=spar,data=dsub,limnla = limnla, control = control)
    pred <- predict(tmp, newdata=data.frame(x=x),pstd=T)
    c(pred$fit,(pred$pstd)^2)
  }
  fit <-apply(output[,1:N],2,mean)
  std <- sqrt(apply(output[,(N+1):(2*N)],2,kmeansq))
  return(list(fit=fit,std=std))
}

##------- 3.drdr ---------##
drdr <- function(x,y,nsub,core=20,alpha=4/5,
                 spar = "v",limnla = c(-10, 3), control = list(),seed=NULL){
  registerDoParallel(cores = core)
  kmeansq <- function(x){
    sum(x)/(length(x))^2
  }
  N <-length(x)
  k <- floor(N/nsub)	
  bounds <- data.frame(lb=c(1+(1:k-1)*nsub),ub=c((1:(k-1))*nsub,N))
  d <- data.frame(x=x, y=y)
  set.seed(seed)
  d <- d[sample(1:N),]
  output <- foreach(i=1:k,.combine = rbind) %dopar%{
    dsub <- d[(bounds[i,1]):(bounds[i,2]),]
    tmp <- ssr(y~x,cubic(x),spar=spar,data=dsub,limnla = limnla, control = control)
    # lambda adjustment
    newlambda <- tmp$rkpk.obj$nlaht-alpha*log10(k)
    tmp1 <- ssr(y~x,cubic(x),data=dsub,spar=method,limnla=c(newlambda,newlambda))
    pred <- predict(tmp1, newdata=data.frame(x=x),pstd=T)
    c(pred$fit,(pred$pstd)^2)
  }
  fit <-apply(output[,1:N],2,mean)
  std <- sqrt(apply(output[,(N+1):(2*N)],2,kmeansq))
  return(list(fit=fit,std=std))
}

##------- 4.sdr ---------##
sdr <- function(x,y,nsub,core=20,
                spar = "v",limnla = c(-10, 3), control = list()){
  registerDoParallel(cores = core)
  N <- length(x)
  k <- floor(N/nsub)	
  bounds <- data.frame(lb=c(1+(1:k-1)*nsub),ub=c((1:(k-1))*nsub,N))
  d <- data.frame(x=sort(x), y=y[order(x)])
  output <- foreach(i=1:k,.combine = rbind) %dopar%{
    dsub <- d[(bounds[i,1]):(bounds[i,2]),]
    tmp <- ssr(y~x,cubic(x),spar=spar,data=dsub,limnla = limnla, control = control)
    pred <- predict(tmp)
    cbind(fit=pred$fit,std=pred$pstd)
  }
  return(data.frame(output))
}

##------- 5.osdr ---------##
osdr <- function(x,y,nsub,core=20,overlap=0.1,
                 spar = "v",limnla = c(-10, 3), control = list()){
  registerDoParallel(cores = core)
  N <- length(x)
  k <- floor(N/nsub)	
  J <- ceiling(nsub*overlap/2)
  d <- data.frame(x=sort(x), y=y[order(x)])
  bounds <- NULL
  for (kk in 1:k) {
    #print(kk)
    lb <- ifelse (kk==1,1+(kk-1)*nsub,1+(kk-1)*nsub-J)
    ub <- ifelse (kk==k,N,kk*nsub+J)
    bounds <- rbind(bounds,c(lb,ub))
    if (kk>1) {
    }
    lbold <- lb; ubold <- ub; 
  }
  overlap.loc <- cbind(bounds[-1,1],bounds[-k,2])
  output <- foreach(i=1:k) %dopar%{
    dsub <- d[(bounds[i,1]):(bounds[i,2]),]
    tmp <- ssr(y~x,cubic(x),spar=spar,data=dsub,limnla = limnla, control = control)
    pred <- predict(tmp)
    cbind(fit=pred$fit,std=pred$pstd)
  }
  lower <-lapply(output[-k],tail,2*J)
  upper <-lapply(output[-1],head,2*J)
  non.over.id<-data.frame(lb=c(1,rep(2*J+1,k-1)),ub=c(nsub-J,rep(nsub,k-2),nsub+J))
  non.overlap <-foreach(i=1:k) %dopar%{
    output[[i]][non.over.id[i,1]:non.over.id[i,2],]
  }
  overlap.fit <- foreach(i=1:(k-1)) %dopar%{
    x.overlap <- x[overlap.loc[i,1]:overlap.loc[i,2]]
    #linear weight
    #w <-1-(x.overlap-x.overlap[1])/(x.overlap[2*J]-x.overlap[1]) 
    #cosine weight
    #w <-0.5+0.5*cos((x.overlap-x.overlap[1])/(x.overlap[2*J]-x.overlap[1])*pi)
    #sine weight
    w <-sin((x.overlap-x.overlap[1])/(x.overlap[2*J]-x.overlap[1])*2*pi)/(2*pi)-(x.overlap-x.overlap[2*J])/(x.overlap[2*J]-x.overlap[1])
    stitch <- w*lower[[i]]+(1-w)*upper[[i]]
  }
  final.fit <- NULL
  for (i in 1:(k-1)){
    final.fit <- rbind(final.fit,non.overlap[[i]],overlap.fit[[i]])
  }
  final.fit <- rbind(final.fit,non.overlap[[k]])
  rownames(final.fit)<-NULL
  final.fit <- data.frame(final.fit)
  return(final.fit)
}

##------- 6.ubs ---------##
ubs <- function(x,y,method,alpha=1) {
  tmp <- try(ssanova(y~x, method=method,alpha=alpha))
  if (class(tmp)=="ssanova") pred <- predict(tmp, newdata=data.frame(x=x),se.fit=T)
  if (class(tmp)!="ssanova") pred <- list(fit=rep(NA, length(y)),se.fit=rep(NA, length(y)))
  return(pred)
}

##------- 7.abs ---------##
abs <- function(x,y,alpha=1,method="v"){
  library(gss)
  N <- length(y)
  my.y.slice <- hist(y,breaks="Scott", plot=F)$breaks
  my.sample <- c(1, N)
  nslice <- length(my.y.slice)-1
  nbasis <- max(30, ceiling(10*N^(2/9)))
  nobs.slice <- ceiling(nbasis/nslice)
  for(kk in 1:(nslice-1)){  
    my.which <- which( my.y.slice[kk]<=y & y<my.y.slice[kk+1])
    if( length(my.which) >= nobs.slice){
      my.sample <- union(my.sample,sample(my.which, nobs.slice ))
    } else {    
      my.sample <- sort(union(my.sample, my.which))
    }
  }
  tmp <- try(ssanova(y~x,id.basis=my.sample,alpha=alpha,method=method))
  if (class(tmp)=="ssanova") pred <- predict(tmp, newdata=data.frame(x=x),se.fit=T)
  if (class(tmp)!="ssanova") pred <- list(fit=rep(NA, length(y)),se.fit=rep(NA, length(y)))
  return(pred)
}

